import React, { useCallback, useState, useMemo } from 'react';
import { StyleSheet, View, Image as OriginImage, ActivityIndicator, } from 'react-native';
import ContentLoader from 'react-content-loader';
import { Path } from 'react-native-svg';
// import * as Progress from 'react-native-progress';
import { useOverride, useMemoStyles } from '@react-native-cask-ui/theme';
const defaultStyles = StyleSheet.create({
    root: {
        overflow: 'hidden',
    },
    image: {
        flex: 1,
        width: undefined,
        height: undefined,
    },
});
const fixedStyles = StyleSheet.create({
    loadingWrapper: {
        position: 'absolute',
        top: 0,
        left: 0,
        right: 0,
        bottom: 0,
        justifyContent: 'center',
        alignItems: 'center',
    },
    loadingProgressWrapper: {
        opacity: 0.7,
    },
});
const Image = React.memo(props => {
    const { props: overridedProps, styles } = useOverride('Image', props);
    const { ImageRenderer = OriginImage, source, fill, width, height, placeholderWidth, placeholderHeight, retryable, placeholderType, loaderPrimaryColor, loaderSecondaryColor, 
    // progressColor,
    onLoad, onError, ...otherProps } = overridedProps;
    const { uri } = source;
    const loadable = !!uri;
    const [retry, setRetry] = useState(0);
    const [loading, setLoading] = useState(true);
    const [waiting, setWaiting] = useState(false);
    const [progress, setProgress] = useState(0);
    const [realWidth, setRealWidth] = useState();
    const [realHeight, setRealHeight] = useState();
    const [wrapperWidth, setWrapperWidth] = useState();
    const [wrapperHeight, setWrapperHeight] = useState();
    const handleProgress = useCallback(e => {
        if (placeholderType !== 'progress')
            return;
        const { loaded, total } = e.nativeEvent;
        const newProgress = loaded / total;
        // RN is a bit buggy with these events, sometimes a loaded event and then a few
        // 100% progress – sometimes in an infinite loop. So we just assume 100% progress
        // actually means the image is no longer loading
        if (progress !== 1 && progress !== newProgress) {
            setLoading(newProgress < 1);
            setProgress(newProgress);
        }
    }, [placeholderType, progress]);
    const handleLoad = useCallback(e => {
        if (onLoad)
            onLoad(e);
        const { width: sourceWidth, height: sourceHeight } = e.nativeEvent.source || {};
        setLoading(false);
        setRealWidth(sourceWidth);
        setRealHeight(sourceHeight);
    }, [onLoad]);
    const handleError = useCallback(error => {
        if (onError)
            onError(error);
        setLoading(true);
        setWaiting(true);
        if (retry < 30) {
            setTimeout(() => {
                // reload and reset progress
                setRetry(retry + 1);
                setWaiting(false);
                setProgress(0);
            }, 2000);
        }
    }, [onError, retry]);
    const handleLayout = useCallback((event) => {
        const { width: sourceWidth, height: sourceHeight } = event.nativeEvent.layout;
        setWrapperWidth(sourceWidth);
        setWrapperHeight(sourceHeight);
    }, []);
    // render
    const wrapperStyle = useMemo(() => ({
        flex: fill ? 1 : undefined,
        width: width || (height && realWidth && realHeight ? (realWidth / realHeight) * height : null) || placeholderWidth,
        height: height || (width && realHeight && realWidth ? (realHeight / realWidth) * width : null) || placeholderHeight,
    }), [fill, width, height, realWidth, realHeight, placeholderWidth, placeholderHeight]);
    const finalStyle = useMemoStyles([defaultStyles.root, styles.root, wrapperStyle]);
    const finalImageStyle = useMemoStyles([defaultStyles.image, styles.image]);
    // use different uri to clear cache on Android.
    // cache may cause onError loop infinitely.
    const newUriSource = useMemo(() => {
        const clearCache = `clearcache=${Date.now()}`;
        if (uri && retry >= 0) {
            return {
                ...source,
                uri: !uri.includes('?') ? `${uri}?${clearCache}` : `${uri}&${clearCache}`,
            };
        }
        return undefined;
    }, [uri, retry]);
    const newSource = uri ? newUriSource : source;
    let newWidth = wrapperWidth || width || placeholderWidth;
    let newHeight = wrapperHeight || height || placeholderHeight;
    // auto calculate width or height
    // if the width or height is zero, image will not be download from source.
    // therefore, must set both placeholderWidth and placeholderHeight if width or height is not set.
    if (realWidth && realHeight) {
        if (width && !height) {
            newHeight = (width / realWidth) * realHeight;
        }
        else if (!width && height) {
            newWidth = (height / realHeight) * realWidth;
        }
    }
    return (<View style={finalStyle} onLayout={handleLayout}>
      {!waiting && (
    /* @ts-ignore */
    <ImageRenderer style={finalImageStyle} source={newSource} onProgress={handleProgress} {...otherProps} onLoad={handleLoad} {...(loadable && retryable
        ? {
            onError: handleError,
        }
        : {})}/>)}
      {loadable && loading && (<View style={fixedStyles.loadingWrapper}>
          {placeholderType === 'loader' && (<ContentLoader foregroundColor={loaderPrimaryColor || '#f7f7f7'} backgroundColor={loaderSecondaryColor || '#e9e9e9'} width={newWidth} height={newHeight}>
              <Path d={`M0 0 h${newWidth || 0} v${newHeight || 0} h-${newWidth || 0}z`}/>
              
            </ContentLoader>)}
          {placeholderType === 'spinner' && <ActivityIndicator />}
          {placeholderType === 'progress' && (<View style={fixedStyles.loadingProgressWrapper}>
              
            </View>)}
        </View>)}
    </View>);
});
export default Image;
